@@ -1,12 +1,19 @@
 import json, cStringIO
-from libpathod import pathoc, test, version
+from libpathod import pathoc, test, version, pathod
 import tutils
 
+def test_response():
+    r = pathoc.Response("1.1", 200, "Message", {}, None)
+    assert repr(r)
 
-class TestDaemon:
+
+class _TestDaemon:
+    ssloptions = pathod.SSLOptions()
     @classmethod
     def setUpAll(self):
         self.d = test.Daemon(
+            ssl=self.ssl,
+            ssloptions=self.ssloptions,
             staticdir=tutils.test_data.path("data"),
             anchors=[("/anchor/.*", "202")]
         )
@@ -19,11 +26,48 @@ class TestDaemon:
         self.d.clear_log()
 
     def test_info(self):
-        c = pathoc.Pathoc("127.0.0.1", self.d.port)
+        c = pathoc.Pathoc(
+            "127.0.0.1",
+            self.d.port,
+            ssl = self.ssl
+        )
         c.connect()
-        _, _, _, _, content = c.request("get:/api/info")
-        assert tuple(json.loads(content)["version"]) == version.IVERSION
+        r = c.request("get:/api/info")
+        assert tuple(json.loads(r.content)["version"]) == version.IVERSION
 
+
+class TestDaemonSSL(_TestDaemon):
+    ssl = True
+    ssloptions = pathod.SSLOptions(request_client_cert=True)
+    def test_sni(self):
+        c = pathoc.Pathoc(
+            "127.0.0.1",
+            self.d.port,
+            ssl = True,
+            sni = "foobar.com"
+        )
+        c.connect()
+        c.request("get:/p/200")
+        r = c.request("get:/api/log")
+        d = json.loads(r.content)
+        assert d["log"][0]["request"]["sni"] == "foobar.com"
+
+    def test_clientcert(self):
+        c = pathoc.Pathoc(
+            "127.0.0.1",
+            self.d.port,
+            ssl = True,
+            clientcert = tutils.test_data.path("data/clientcert/client.pem")
+        )
+        c.connect()
+        c.request("get:/p/200")
+        r = c.request("get:/api/log")
+        d = json.loads(r.content)
+        assert d["log"][0]["request"]["clientcert"]["keyinfo"]
+
+
+class TestDaemon(_TestDaemon):
+    ssl = False
     def tval(self, requests, showreq=False, showresp=False, explain=False, hexdump=False, timeout=None, ignorecodes=None, ignoretimeout=None):
         c = pathoc.Pathoc("127.0.0.1", self.d.port)
         c.connect()
@@ -43,6 +87,10 @@ class TestDaemon:
             )
         return s.getvalue()
 
+    def test_ssl_error(self):
+        c = pathoc.Pathoc("127.0.0.1", self.d.port, ssl = True)
+        tutils.raises("ssl handshake", c.connect)
+
     def test_ignorecodes(self):
         assert "200" in self.tval(["get:'/p/200:b@1'"])
         assert "200" not in self.tval(["get:'/p/200:b@1'"], ignorecodes=[200])
@@ -85,3 +133,19 @@ class TestDaemon:
         assert "foo" in self.tval(["+%s"%d], showreq=True)
         assert "File" in self.tval(["+/nonexistent"])
 
+    def test_connect_fail(self):
+        to = ("foobar", 80)
+        c = pathoc.Pathoc("127.0.0.1", self.d.port)
+        r, w = cStringIO.StringIO(), cStringIO.StringIO()
+        tutils.raises("connect failed", c.http_connect, to, w, r)
+        r = cStringIO.StringIO(
+            "HTTP/1.1 500 OK\r\n"
+        )
+        tutils.raises("connect failed", c.http_connect, to, w, r)
+        r = cStringIO.StringIO(
+            "HTTP/1.1 200 OK\r\n"
+        )
+        c.http_connect(to, w, r)
+
+
+
